#include "threadmgr.h"

ThreadMgr::ThreadMgr(const QString& clu_file) : _cluFile(clu_file), _incomplete(true)
{
}

int ThreadMgr::RunKMeansPlusPlus(int num_records, int k_min, int k_max, int num_samples,
                  Scalar* records, int num_attempts)
{
    _numRecords = num_records;
    _numSamples = num_samples;
    _numThreadsRemaining = k_max - k_min + 1;
    _threads = QVector<QThread*>(k_max+1);
    bool is_only_cluster = (_numThreadsRemaining==1);
    Q_ASSERT(_clusters.size() > 0);
    QString msg;
    for (int k=k_min; k<=k_max; ++k)
    {
        QThread* thd = new QThread(this);
        KMeansTO* to = new KMeansTO(this, num_records, k, num_samples, records, num_attempts, is_only_cluster);
//        KMeansTO* to = new KMeansTO(num_records, k, num_samples, records, num_attempts);
        to->moveToThread(thd);
//        connect(to, SIGNAL(Finished(int,const int*,const Scalar*)),
//                this, SLOT(KMeansFinished(int,const int*,const Scalar*)), Qt::QueuedConnection);
            //Signals and slots not working here for some reason!!
        _threads[k] = thd;
        thd->start();
        connect(this, SIGNAL(Start()), to, SLOT(Start()), Qt::QueuedConnection);
        emit Start();
        disconnect(this, SIGNAL(Start()), to, SLOT(Start()));
        connect(thd, SIGNAL(finished()), thd, SLOT(deleteLater()));
        msg = QString("Thread %1 launched").arg(k);
        cerr << msg.toStdString() << endl;
    }

    while ( IsIncomplete() )  Sleeper::sleep_us(100000);
    return _result;
}

void ThreadMgr::KMeansFinished(int k, const int* assignments, const Scalar* centers)
{
    QMutexLocker lock(&_mutex);
    QString msg = QString("KMeans++ finished for clustering %1").arg(k);
    cerr << msg.toStdString() << endl;
    Q_ASSERT(k!=0);
    _clusters.insert(k, ClusPair(assignments, centers));
    _threads.at(k)->quit();
    if (--_numThreadsRemaining == 0)
    {
        _result = WriteCluster();
        lock.unlock();
        SetComplete();
    }
    msg = QString("Number of threads remaining: %1").arg(_numThreadsRemaining);
    cerr << msg.toStdString() << endl;
}

int ThreadMgr::WriteCluster()
{
    for (QMap<int, ClusPair>::const_iterator it = _clusters.begin(); it!=_clusters.end(); ++it)
    {
        int k = it.key();
        Q_ASSERT(k!=0);
        const int* assignments = it->first;
        const Scalar* centers = it->second;
        int current_result = km::WriteCluFile(_cluFile, k, assignments, centers, _numRecords, _numSamples);
        if (current_result)
            return current_result;
    }
    return 0;
}
