#include <QtCore/QCoreApplication>
#include "KlustaKwik.h"

int main(int argc, char **argv)
{
    float Score;
    float BestScore = HugeScore;
    int p, i;
    SetupParams(argc, argv);
    clock_t Clock0;
    KK K1; // main KK class, for all data
    K1.penaltyMix = PenaltyMix;

    Clock0 = clock(); // start timer

    K1.LoadData(); // load .fet file

    // Seed random number generator
    srand(RandomSeed);

    // open distance dump file if required
    if (DistDump) Distfp = fopen("DISTDUMP", "w");

    // start with provided file, if required
    if (*StartCluFile)
    {
        Output("Starting from cluster file %s\n", StartCluFile);
        BestScore = K1.CEM(StartCluFile, 1, 1);
        Output("%d->%d Clusters: Score %f\n\n", K1.nStartingClusters, K1.nClustersAlive, BestScore);
        for(p=0; p<K1.nPoints; ++p)
            K1.BestClass[p] = K1.Class[p];
        SaveOutput(K1.BestClass);
    }


    // loop through numbers of clusters ...
    for(K1.nStartingClusters=MinClusters; K1.nStartingClusters<=MaxClusters; K1.nStartingClusters++)
        for(i=0; i<nStarts; ++i)
        {
            // do CEM iteration
            Output("Starting from %d clusters...\n", K1.nStartingClusters);
            Score = K1.Cluster();

            Output("%d->%d Clusters: Score %f, best is %f\n", K1.nStartingClusters, K1.nClustersAlive, Score, BestScore);

            if (Score < BestScore)
            {
                Output("THE BEST YET!\n");
                // New best classification found
                BestScore = Score;
                for(p=0; p<K1.nPoints; ++p)
                    K1.BestClass[p] = K1.Class[p];
                SaveOutput(K1.BestClass);
            }
            Output("\n");
        }

    SaveOutput(K1.BestClass);

    Output("That took %f seconds.\n", (clock()-Clock0)/(float) CLOCKS_PER_SEC);

    if (DistDump) fclose(Distfp);

    return 0;
}
