#include "analysistobase.h"
#include "../memrep/wavedata.h"

AnalysisTOBase::AnalysisTOBase(MatCPtr waves, int num_records,
                               const std::string& file_name)
    : _analysisData(nullptr), _inputFullPath(file_name), _numRecords(num_records), _waves(waves)
{
}
AnalysisTOBase::AnalysisTOBase(WaveData* wave_data, int num_records,
                               const std::string& file_name)
    : _inputFullPath(file_name), _numRecords(num_records), _waveData(wave_data)
{
}
AnalysisTOBase::~AnalysisTOBase()
{
    _waves.reset(); //deleted != out of scope
    if (_analysisData)
        if (auto pair = static_cast< std::pair<wv::TRANSFORM,MatPtr>* >(_analysisData))
        {
            pair->second.reset();
            delete pair;
        }
}

void AnalysisTOBase::MakeProgressBar(const std::string &label)
{
//    wv::CLUSTER_ID dummy;
//    FileBase::TempFileComponents(wfm_file, _userDir, _shortInputFileName, _transform, dummy, _electrode);
//    std::string pbar_label = _shortInputFileName + ", " + label + " " + Type();
    _progressBar = new ProgressBar(label, 0, _numRecords);
    pg().AddProgressBar(_progressBar);
    connect(this, SIGNAL(SetPBarValue(int)), _progressBar, SLOT(SetValue(int)), Qt::QueuedConnection);
}

void AnalysisTOBase::AnalysisFinished(const void* analysis) const
{
    pg().RemoveProgressBar(_progressBar);
    emit Finished(analysis);
}

void AnalysisTOBase::AnalysisFinished(void* analysis)
{
    pg().RemoveProgressBar(_progressBar);
    //The ProgressBar will be deleted when the ProgressGui is, since it's its child.
    _analysisData = analysis;
    emit Finished(analysis);
}
