#ifndef ANALYSISTOBASE_H
#define ANALYSISTOBASE_H

#include "../globals/globals.h"
#ifdef USE_GSL
#include "gsl_utils/mgsl.h"
#endif

#include "../memrep/parameters/paramsbase.h"
#include "../gui/mdi/progressgui.h"
#include "../gui/widgets/progressbar.h"
#include "../globals/log.h"

class WaveData;

//TO for thread object.  The matrix it creates, transform, is allocated here but freed in WaveData
class AnalysisTOBase : public QObject
{
    Q_OBJECT

    public:
        AnalysisTOBase(MatCPtr wave_mat, int num_records, const std::string& input_file_name);
        AnalysisTOBase(WaveData* data, int num_records, const std::string& input_file_name);
        //Note that the input file name structure is *different* for transforms and cluster methods--naturally,
            //they are different files coming from different places
        virtual ~AnalysisTOBase();

        void StartTimer() { _time.start(); }

        virtual std::string Name() const = 0;
        int TimeElapsed() const { return _time.elapsed(); }
        int NumRecords() const { return _numRecords; }

    signals:
        void Finished(void* analysis) const; //No analysis should ever be changed after it has
        void Finished(const void* analysis) const; //No analysis should ever be changed after it has
            //been completed
        void AddProgressBar(QProgressBar* progress_bar);
        void RemoveProgressBar(QProgressBar* progress_bar);
        void SetPBarValue(int i);

    public slots:
        virtual void Start() = 0;

    protected:
        void MakeProgressBar(const std::string& label);
        virtual void AnalysisFinished(void* analysis) = 0;
        virtual void AnalysisFinished(const void* analysis) const = 0;
            //It's a wv_matrix* for transforms, a Classification* for cluster methods

        void SetParams(ParamsCPtr pars) { _pars = pars; }

        virtual std::string InputFileName() const = 0;
        const std::string& InputFullPath() const { return _inputFullPath; }
        ParamsCPtr Params() const { return _pars; }
        MatCPtr Waves() const { return _waves; }
        WaveData* WData() const { return _waveData; }
        std::string Type() const { return "transform"; }
        virtual wv::TRANSFORM Transform() const = 0; // *All* analyses either are a transform, or are based
            //of a particular transform

    private:
        void* _analysisData; //Basically so that Matrix pointers from transforms can be appropriately
            //deleted.
        std::string _inputFullPath; //Every analysis is associated with a particular input file
        int _numRecords;
        ParamsCPtr _pars;
#ifndef Q_OS_WIN
        union
        {
#endif
            WaveData* _waveData; //Special member for WaveData (for manual clustering) because it has to be non-const
            MatCPtr _waves; //For transform analyses
#ifndef Q_OS_WIN
        };
#endif
        ProgressBar* _progressBar;
        QTime _time;
};

#endif // ANALYSISTOBASE_H
