#ifndef BATCH_H
#define BATCH_H

#include "../globals/globals.h"
#include "../memrep/clustermediator.h"
#include "../file/input/waves/waveformsinbase.h"
#include "threads/mgr/clustermgr.h"

class Batch : public QObject
{
    Q_OBJECT

    public:
        Batch(const std::list<std::string>& batch_list);
        ~Batch();

        void Cancel();
        void Run();

    signals:
        void BatchComplete();

    private:
#ifndef Q_OS_WIN
        Batch(const Batch&) = delete;
        Batch& operator=(const Batch&) = delete;
#else
        Batch(const Batch&);
        Batch& operator=(const Batch&);
#endif
        void Process();

        StringList _batchList; //Gets decreased as files are processed.
        ClusterMediator& _cmed;
#ifndef Q_OS_WIN
        std::atomic<bool> _isRunning;
#else
        volatile bool _isRunning;
#endif
        std::thread* _thread;
};

#endif // BATCH_H
