#ifndef CLUSMETHODTOBASE_H
#define CLUSMETHODTOBASE_H

#include "../../globals/log.h"
#include "../analysistobase.h"
#include "../../file/input/mclufilein.h"
#include "../../memrep/parameters/cluster/clusterparamsbase.h"
#include "../dunn.h"

class ClusMethodTOBase : public AnalysisTOBase
{
    Q_OBJECT

    public:
        static ClusMethodTOBase* Create(wv::CLUSTER_ID id, wv::TRANSFORM transform, wv::CLUSTER_METHOD cluster_method,
                                    MatCPtr data, const std::string& wfm_file);
        static ClusMethodTOBase* Create(wv::CLUSTER_ID id, wv::CLUSTER_METHOD cluster_method,
                                    WaveData* data, const std::string& wfm_file);
            //Need const and non-const versions because WaveData is non-const
        virtual ~ClusMethodTOBase() override;

        wv::CLUSTER_ID ID() const { return _cid; }
        virtual wv::CLUSTER_METHOD Method() const = 0;

        virtual void SetWindow(int window_begin, int window_end);
            //NB, ManualTO ignores the classification window
        std::string Type() const { return "cluster_method"; }

    protected:
        ClusMethodTOBase(wv::CLUSTER_ID id, wv::TRANSFORM transform, WaveData* wave_data, int num_records,
                         const std::string& wfm_file);
        ClusMethodTOBase(wv::CLUSTER_ID id, wv::TRANSFORM transform, MatCPtr data, int num_records,
                         const std::string& wfm_file);
            //Note that for transforms, the input file is the *data* file; here, it's the *wfm* file which is *generated* by the transform.
            //It is the full path and suffixes, e.g., Users/matt/temp/~ch_100419a.haar.wfm.1

        virtual void AnalysisFinished(void* classification) override;
        virtual void AnalysisFinished(const void* classification) const override;
        void Cluster(QProcess* clus_proc, const StringList& proc_args);
            //Used by all except manual clustering.
        int Electrode() const { return _electrode; }
        virtual wv::TRANSFORM Transform() const override { return _transform; }
        int WindowBegin() const { return _windowBegin; }
        int WindowEnd() const { return _windowEnd; }

        virtual std::string InputFileName() const override;
        std::string InputPathStub(); //This returns the path of the input wfm file, excluding the trailing '.wfm.<n>'

    protected slots:
        virtual void LoadClassification(int exit_code);

    private slots:
        void ReadStandardOutput();
        void ReadStandardError();
        void StateChange(QProcess::ProcessState);
        void Error(QProcess::ProcessError);

    private:
        void Setup();

        const wv::CLUSTER_ID _cid; //Uniquely identifies this analysis, used for associating them
            //with the WaveGui

        int _electrode;
        std::string _userDir,
                _shortInputFileName;
        wv::TRANSFORM _transform;
            //Needed to identify the temp wfm file

        QProcess* _clusProc;
        int _windowBegin, _windowEnd; //The process will use fseek to find the right records

        MCluFileIn* _mcluFile;
};

Q_DECLARE_METATYPE(QProcess::ProcessState)
Q_DECLARE_METATYPE(QProcess::ProcessError)

#endif // CLUSMETHODTOBASE_H
