#ifndef MGSL_H
#define MGSL_H

#include "../../globals/globals.h"
#include "../threads/mgr/covthreadmgr.h"
#include "../threads/mgr/radix2mgr.h"

#ifdef HAS_CUDA
#include "../../../../cumath/cumath.h"
#define IDX2C(i,j,ld) (((j)*(ld))+(i))
#endif

namespace mgsl
{
    gsl_matrix* AllocateRadix2(MatCPtr mat);

    void Covariance(const gsl_matrix* mat, gsl_matrix* cov_mat);
    void CovarianceP(const gsl_matrix* mat, gsl_matrix* cov_mat);
#ifdef HAS_CUDA
    void CovarianceCUDA(const gsl_matrix* mat, gsl_matrix* cov_mat);
    void CublasSgemm(CBLAS_TRANSPOSE_t cbtA, CBLAS_TRANSPOSE_t cbtB,
                     double alpha, const gsl_matrix* A, const gsl_matrix* B,
                     double beta, gsl_matrix* C);
#endif

    double Dist2(const gsl_vector* v1, const gsl_vector* v2);

    void PCATransform(const gsl_matrix* mat, gsl_matrix* trans_mat,
                      gsl_matrix* eigenvecs = nullptr, gsl_vector* eigenvals = nullptr,
                      bool use_cuda = false);
        //Rotates the matrix in accordance
        //with the eigenvectors generated by PrinComp et al. trans_mat needs to be the same size as mat
        //**and must be initialized to 0*** e.g. with gsl_matrix_calloc
        //If eigenvecs and/or eigenvals are supplied, they must be pre-allocated by the calling function

//    void PCATransformCUDA(const gsl_matrix* mat, gsl_matrix* trans_mat,
//                      gsl_matrix* eigenvecs = nullptr, gsl_vector* eigenvals = nullptr);

    void PointsToNLine(const gsl_vector* line_pt1, const gsl_vector* line_pt2, const gsl_matrix* pts,
                        gsl_vector* projns_on_line, gsl_vector* dists_to_line);
        //line_pt1 and line_pt2 are any two points on an n-dimensional line.  pt is an n-dimensional point.
        //Returns by reference the norm of the projection of the point onto the line (distance to line_pt1),
        //and the distance to the line.
        //  proj_on_line and dist_to_line are assumed to *already have been initilized*
    void PrinComp(const gsl_matrix* mat,
                  gsl_matrix* eigenvecs, gsl_vector* eigenvals, bool use_cuda = false);
}

#endif // MGSL_H
