#ifndef CONTAINSMGR_H
#define CONTAINSMGR_H

#include "threadmgr.h"
#include "../../../memrep/drawing/circle.h"

//This simply determines which records are contained in the circle in ContainsSpec and which aren't.  Parallelized
//because esp. for ellipses this needs several dozen operations per record.
class ContainsMgr : public ThreadMgr
{
    public:
        struct ContainsSpec
        {
            ContainsSpec(const Circle* circle_) : circle(circle_),
                i_start(-1), i_end(-1), sample_x(-1), sample_y(-1),
                time_begin(-1), time_end(-1), time_restricted(-1), timestamps(nullptr),
                transform_x(nullptr), transform_y(nullptr)
            {}
            const Circle* circle;
            size_t i_start, i_end;
            int sample_x, sample_y,
                time_begin, time_end;
            bool time_restricted;
            const wv_vector* timestamps;
            const wv_matrix* transform_x, * transform_y;
        };

        explicit ContainsMgr(ContainsSpec& contains_spec, wv_vector_int* vcluster);
            //vcluster must already be allocated

        virtual WorkerTO* CreateWorkerTO(int thd_num) override;

    private:
        ContainsSpec _containsSpec;
        int _recordsPerThread;
        wv_vector_int* _vCluster; //The cluster assignments for each record
};

#endif // CONTAINSMGR_H
