#include "covthreadmgr.h"

CovThreadMgr::CovThreadMgr(const wv_matrix* input_mat, wv_matrix* cov_mat)
    : ThreadMgr(QThread::idealThreadCount(), 25000),
      _inputMat(input_mat), _covMat(cov_mat)
{
    int size1 = input_mat->size1,
            size2 = input_mat->size2;
    _columnMeans = wv_vector_alloc(size2);
    for (int j=0; j<size2; ++j)
    {
        wv_vector_const_view v = wv_matrix_const_column(input_mat, j);
        wv::float_t m = wv_stats_mean(v.vector.data, v.vector.stride, size1);
        wv_vector_set(_columnMeans, j, m);
    }
}
CovThreadMgr::~CovThreadMgr()
{
    if (_columnMeans) wv_vector_free(_columnMeans);
}

WorkerTO* CovThreadMgr::CreateWorkerTO(int thd_num)
{
    size_t num_rows = _inputMat->size1,
        num_cols = _inputMat->size2;

    std::list<CovarianceTO::CovSpec> cov_specs;
    int num_threads = NumThreads();
    for (size_t j1 = 0; j1 < num_cols; ++j1)
        for (size_t j2 = j1; j2 < num_cols; ++j2)
        {
            if (j2%num_threads != (unsigned)thd_num) continue;
            wv_vector_const_view v1 = wv_matrix_const_column(_inputMat, j1),
                v2 = wv_matrix_const_column(_inputMat, j2);

            wv::float_t mean1 = wv_vector_get(_columnMeans, j1),
                    mean2 = wv_vector_get(_columnMeans, j2);

            CovarianceTO::CovSpec cov_spec =
            {
                j1, j2,
                v1.vector.data, v2.vector.data,
                mean1, mean2,
                v1.vector.stride, v2.vector.stride
            };
            cov_specs.push_back(cov_spec);
        }

    CovarianceTO* to = new CovarianceTO(thd_num, cov_specs, num_rows, _covMat);
    return to;
}
