#include "fouriermgr.h"
#include "../worker/fourierworkerto.h"

FourierMgr::FourierSpec::FourierSpec(const wv_matrix* waves_)
    : i_start(-1), i_end(-1), waves(waves_)
{
    transform = wv_matrix_alloc(waves->size1, waves->size2);
    wv_matrix_memcpy(transform, waves);
}


FourierMgr::FourierMgr(FourierSpec& fourier_spec) : ThreadMgr(QThread::idealThreadCount(), 25000),
    _fourierSpec(fourier_spec)
{
    int num_records = _fourierSpec.waves->size1;
    _recordsPerThread = num_records / NumThreads();
}

WorkerTO* FourierMgr::CreateWorkerTO(int thd_num)
{
    int i_start = thd_num*_recordsPerThread;
    _fourierSpec.i_start = i_start;
    _fourierSpec.i_end = (thd_num==NumThreads()-1) ? _fourierSpec.waves->size1 : i_start+_recordsPerThread;
    FourierWorkerTO* to = new FourierWorkerTO(thd_num, _fourierSpec);
    return to;
}
