#include "haarmgr.h"
#include "../worker/haarworkerto.h"

HaarMgr::HaarSpec::HaarSpec(const wv_matrix* waves_)
    : i_start(-1), i_end(-1), waves(waves_)
{
    wv_matrix* transform_pre = wv_matrix_alloc(waves->size1, waves->size2);
    wv_matrix_memcpy(transform_pre, waves);
    MatPtr transform_ptr = std::shared_ptr<wv_matrix>(transform_pre, wv_matrix_free);
#ifdef USE_GSL
    transform = mgsl::AllocateRadix2(transform_ptr);
#else
    transform = msl::AllocateRadix2( transform_ptr.get() );
#endif
}


HaarMgr::HaarMgr(HaarSpec& haar_spec) : ThreadMgr(QThread::idealThreadCount(), 25000),
    _haarSpec(haar_spec)
{
    int num_records = _haarSpec.waves->size1;
    _recordsPerThread = num_records / NumThreads();
}

WorkerTO* HaarMgr::CreateWorkerTO(int thd_num)
{
    int i_start = thd_num*_recordsPerThread;
    _haarSpec.i_start = i_start;
    _haarSpec.i_end = (thd_num==NumThreads()-1) ? _haarSpec.waves->size1 : i_start+_recordsPerThread;
    _haarSpec.is_centered = session_profile.Par("Session", "Is wavelet centered")=="yes";
    std::string par = session_profile.Par("Session", "Wavelet size (or \"haar\")");
    _haarSpec.size = (par=="haar") ? -1 : boost::lexical_cast<int>(par);
    HaarWorkerTO* to = new HaarWorkerTO(thd_num, _haarSpec);
    return to;
}

