#include "threadmgr.h"
#include "../worker/workerto.h"

ThreadMgr::ThreadMgr(int num_threads, int us_sleep) : _numThreads(num_threads), _usSleep(us_sleep)
{
    for (int i=0; i<_numThreads; ++i)
        _threads.push_back( new QThread(this) );
}
ThreadMgr::~ThreadMgr()
{
    for (auto worker : _workers) delete worker;
    _workers.clear();
}

void ThreadMgr::StartThreads()
{
    MutexLocker lock(_mutex);

    for (int j=0; j<_numThreads; ++j)
    {
        QThread* thd = _threads[j];

        WorkerTO* to = CreateWorkerTO(j);
        _workers.push_back(to);
        to->moveToThread(thd);
        thd->start();

        connect(this, SIGNAL(Start()), to, SLOT(Start()), Qt::QueuedConnection);
        connect(to, SIGNAL(Finished(int, void*)), this, SLOT(ThreadFinished(int, void*)), Qt::DirectConnection);
        emit Start();
        disconnect(this, SIGNAL(Start()), to, SLOT(Start()));
        connect(thd, SIGNAL(finished()), thd, SLOT(deleteLater()));
    }
}
void ThreadMgr::Wait()
{
    while ( IsIncomplete() ) Sleeper::sleep_us(_usSleep);
}

void ThreadMgr::ThreadFinished(int thd_num, void* result) //slot
{
    MutexLocker lock(_mutex);

    OnThreadFinished(result);

    _threads[thd_num]->quit();
}

bool ThreadMgr::IsIncomplete()
{
    MutexLocker lock(_mutex);
    for (int i=0; i<_numThreads; ++i)
        if (_threads[i]->isRunning()) return true;
    return false;
    //This way is slower than simply keeping track of _numThreadsRemaining; but unless this done
    //it appears that the HistThreadMgr destroys the thread before it has quit, especially on machines
    //with good graphics cards--am getting 'thread destroyed while still running' debugger output.
}
