#include "containsto.h"

ContainsTO::ContainsTO(int thd_num, const ContainsMgr::ContainsSpec& contains_spec,
                       wv_vector_int* vcluster)
    : WorkerTO(thd_num), _containsSpec(contains_spec), _vCluster(vcluster)
{
}

void ContainsTO::Start()
{
    //Get data for Contains calculation
    const Circle* circle = _containsSpec.circle;
    int sample_x = _containsSpec.sample_x,
        sample_y = _containsSpec.sample_y,
        time_begin = _containsSpec.time_begin,
        time_end = _containsSpec.time_end;
    bool time_restricted = _containsSpec.time_restricted;
    const wv_vector* timestamps = _containsSpec.timestamps;
    const wv_matrix* transform_x = _containsSpec.transform_x,
                * transform_y = _containsSpec.transform_y;

    //Run the Contains calculation on these records
    size_t i_end = _containsSpec.i_end;
    for (size_t i=_containsSpec.i_start; i<i_end; ++i)
    {
        if (time_restricted)
        {
            wv::float_t rec_time = wv_vector_get(timestamps,i);
            if (rec_time<time_begin || rec_time>time_end)
                continue;
        }

        wv::float_t xval = wv_matrix_get(transform_x, i, sample_x),
                yval = wv_matrix_get(transform_y, i, sample_y);
        if ( circle->Contains(xval, yval) )
            wv_vector_int_set(_vCluster, i, wv::ACCEPT_CLUSTER);
    }

    WorkFinished();
}
