#include "histto.h"

HistTO::HistTO(int thd_num, HistSpec hist_spec) :
    WorkerTO(thd_num), _histSpec(hist_spec)
{
}

void HistTO::Start()
{
    //Get information from _histSpec, mostly for convenience.
    bool has_accepted = _histSpec.has_accepted;
    int num_bins_x = _histSpec.num_bins_x,
        num_bins_y = _histSpec.num_bins_y,
        start_row = _histSpec.start_row,
        end_row = _histSpec.end_row;
    wv::float_t min_x = _histSpec.min_x,
            min_y = _histSpec.min_y,
            max_x = _histSpec.max_x,
            max_y = _histSpec.max_y;
    const boost::dynamic_bitset<>& is_accepted = _histSpec.is_accepted;
    const WaveSpaceGuiSpec& spec = _histSpec.spec;
    const wv_vector* xdata_vec = _histSpec.xdata_vec,
            * ydata_vec = _histSpec.ydata_vec;

    //Allocate and initialize histograms
    wv_histogram2d* hist_accept = wv_histogram2d_alloc(num_bins_x, num_bins_y),
                * hist_unaccept = wv_histogram2d_alloc(num_bins_x, num_bins_y);
    wv_histogram2d_set_ranges_uniform(hist_accept, min_x, max_x, min_y, max_y);
    wv_histogram2d_set_ranges_uniform(hist_unaccept, min_x, max_x, min_y, max_y);

    bool use_transform = spec.HistLimitMode()==WSGSpec::TRANSFORM;
    wv::float_t time_begin = spec.TimeBegin(),
            time_end = spec.TimeEnd();
#ifdef USE_GSL
    if (use_transform)
        for (int i=start_row; i<end_row; ++i)
        {
            if (has_accepted && is_accepted.test(i))
                wv_histogram2d_increment(hist_accept,
                                          wv_vector_get(xdata_vec, i), wv_vector_get(ydata_vec, i));
            else
                wv_histogram2d_increment(hist_unaccept,
                                          wv_vector_get(xdata_vec, i), wv_vector_get(ydata_vec, i));
        }
    else
        for (int i=start_row; i<end_row; ++i)
        {
            wv::float_t timestamp = spec.Timestamp(i);
            if (timestamp >= time_begin && timestamp <= time_end)
            {
                if (has_accepted && is_accepted.test(i))
                    wv_histogram2d_increment(hist_accept,
                                              wv_vector_get(xdata_vec, i), wv_vector_get(ydata_vec, i));
                else
                    wv_histogram2d_increment(hist_unaccept,
                                              wv_vector_get(xdata_vec, i), wv_vector_get(ydata_vec, i));
            }
                //Get it--if the record falls outside the time window, it isn't included in the histogram.
        }
    //A little inelegant to repeat the loop, but the speed gain is probably non-negligible
#else
    if (use_transform)
    {
        if (has_accepted)
            for (int i=start_row; i<end_row; ++i)
                wv_histogram2d_increment( is_accepted.test(i) ? hist_accept : hist_unaccept,
                                          wv_vector_get(xdata_vec, i), wv_vector_get(ydata_vec, i));
        else
            wv_histogram2d_populate(hist_unaccept,
                                    xdata_vec->data+(start_row*xdata_vec->stride), xdata_vec->stride,
                                    ydata_vec->data+(start_row*ydata_vec->stride), ydata_vec->stride,
                                    end_row-start_row);
    }
    else
        for (int i=start_row; i<end_row; ++i)
        {
            wv::float_t timestamp = spec.Timestamp(i);
            if (timestamp >= time_begin && timestamp <= time_end)
            {
                if (has_accepted && is_accepted.test(i))
                    wv_histogram2d_increment(hist_accept,
                                              wv_vector_get(xdata_vec, i), wv_vector_get(ydata_vec, i));
                else
                    wv_histogram2d_increment(hist_unaccept,
                                              wv_vector_get(xdata_vec, i), wv_vector_get(ydata_vec, i));
            }
                //Get it--if the record falls outside the time window, it isn't included in the histogram.
        }
    //A little inelegant to repeat the loop, but the speed gain is probably non-negligible
#endif
    auto result = new std::pair<wv_histogram2d*, wv_histogram2d*>(hist_accept, hist_unaccept);
    WorkFinished(result);
}
