#include "radix2to.h"

Radix2TO::Radix2TO(int thd_num, const Radix2Mgr::Radix2Spec& radix2_spec,
                       wv_matrix* radix2_mat)
    : WorkerTO(thd_num), _radix2Spec(radix2_spec), _radix2Mat(radix2_mat)
{
}

void Radix2TO::Start()
{
    //Get info from spec
    const wv_matrix* mat = _radix2Spec.mat.get();
    int num_cols = _radix2Spec.num_cols;
    const wv::float_t* x_interp = _radix2Spec.x_interp;
    int num_cols_orig = mat->size2;

    gsl_interp_accel* accel = gsl_interp_accel_alloc();

    //Initialize sample number--these do not have to be equally space in principle
    wv::float_t* xa = new wv::float_t[num_cols_orig],
           * ya = new wv::float_t[num_cols_orig];
    for (int i=0; i<num_cols_orig; ++i)
        xa[i] = (wv::float_t)i;

    //Now iterate through the records, linearly interpolating
    gsl_spline* spline = gsl_spline_alloc(gsl_interp_linear, num_cols_orig);
    wv_vector* vec = wv_vector_alloc(num_cols_orig);
    size_t i_end = _radix2Spec.i_end;
    for (size_t i=_radix2Spec.i_start; i<i_end; ++i)
    {
        //Get ya
        wv_matrix_get_row(vec, mat, i);
        for (int j=0; j<num_cols_orig; ++j)
        {
            wv::float_t d = (wv::float_t)wv_vector_get(vec, j);
            ya[j] = d;
        }

        //Initialize spline
        gsl_spline_init(spline, xa, ya, num_cols_orig);

        //Fill up transform
        for (int j=0; j<num_cols; ++j)
        {
            wv::float_t yi = gsl_spline_eval(spline, x_interp[j], accel);
            wv_matrix_set(_radix2Mat, i, j, yi);
        }
    }

    //Free resources
    delete[] xa;
    delete[] ya;
    gsl_spline_free(spline);
    gsl_interp_accel_free(accel);

    WorkFinished();
}
