#include "pca_to.h"

PCA_TO::PCA_TO(MatCPtr waves, const std::string& file_name,
               short electrode, ParamsBase* pars, bool is_raw)
    : TransformTOBase(waves, file_name, electrode, pars, is_raw)
{
//    MakeProgressBar(file_name);
}

void PCA_TO::Start()
{
    const wv_matrix* waves = Waves().get();
    int num_records = waves->size1;

    size_t num_components = boost::lexical_cast<size_t>( session_profile.Par("Session", "Num PCA components") );
//    if (num_components > waves->size2) throw BadParameter("PCA_TO::Start", "Num PCA components");
    if (num_components > waves->size2) num_components = waves->size2;
    wv_matrix* transform = wv_matrix_calloc( num_records, num_components);  // *_calloc initializes to 0
#ifdef HAS_CUDA
    bool use_cuda = session_profile.Par("Session", "Enable CUDA") == "yes";
    mgsl::PCATransform(waves, transform, nullptr, nullptr, use_cuda);
#elif defined(USE_GSL)
    mgsl::PCATransform(waves, transform, nullptr, nullptr, false);
#else
    msl::PCATransform(waves, transform, false, (wv_matrix*)nullptr, (wv_vector*)nullptr);
#endif

    MatPtr transform_ptr = MatPtr(transform, wv_matrix_free);
    auto pair = new std::pair<wv::TRANSFORM, MatPtr>( Transform(), transform_ptr );
    AnalysisFinished(pair);
}
