#include "filebase.h"

std::string FileBase::_usersDir,
    FileBase::_userName,
    FileBase::_userDataDir,
    FileBase::_userWSDataDir;

FileBase::FileBase(const std::string& full_path, QIODevice::OpenMode mode, bool is_binary)
    : _mode(mode), _isBinary(is_binary), _file()
{
    size_t pos = full_path.find_last_of('/');
    _dir = full_path.substr(0, pos);
    _name = full_path.substr(pos+1);

    Open();
}

FileBase::~FileBase()
{
    if (_file) fclose(_file);
}

std::string FileBase::MakeTempFileName(const std::string& users_dir, const std::string& user_name, const std::string& input_file_name,
                             wv::TRANSFORM transform, wv::CLUSTER_ID cid, const std::string& type, int electrode)
{
    std::string tr = wv::TransformName(transform);
    return users_dir + "/" + user_name + "/temp/~" + input_file_name + "." + tr + "."
            + std::to_string(cid) + "." + type + "." + std::to_string(electrode);
}
std::string FileBase::MakeTempFileName(const std::string& full_path, const std::string& input_file_name,
                             wv::TRANSFORM transform, wv::CLUSTER_ID cid, const std::string& type, int electrode)
{
    std::string user_dir(""),
            user_name("");
    int pos = full_path.find_last_of('/');
    if (pos==-1)
        user_name = full_path;
    else
    {
        user_dir = full_path.substr(0,pos);
        user_name = full_path.substr(pos+1);
    }
    return MakeTempFileName(user_dir, user_name, input_file_name, transform, cid, type, electrode);
}
void FileBase::TempFileComponents(const std::string& temp_file, std::string& user_dir, std::string& name, wv::TRANSFORM& transform,
                            wv::CLUSTER_ID& cid, int& electrode)
{
    StringVec parts = ut::Split<StringVec>(temp_file, "/");
    std::string full_name = "";
    for (auto it=parts.rbegin(); it!=parts.rend(); ++it)
    {
        std::string part = *it;
        if (part.at(0)=='~' && full_name.empty()) full_name = part.substr(1);
        if (part=="temp")
        {
            StringVec vdir = StringVec(parts.begin(), it.base());
            user_dir = boost::join(vdir, "/");
            break;
        }
    }

    StringVec elts = ut::Split<StringVec>(full_name, ".");
    int num_elts = elts.size(); //The filename itself can have .'s in it, so need to be careful
    name = "";
    for (int i=0; i<num_elts-4; ++i)
        name += elts.at(i) + ".";
    name.erase( name.size()-1 ); //Get rid of the last .
    std::string ts = elts.at(num_elts-4);
    transform = wv::TransformEnumVal(ts);
    cid = boost::lexical_cast<int>( elts.at(num_elts-3) );
    electrode = boost::lexical_cast<int>( elts.at(num_elts-1) ); //"wfm" or "clu" is element n-2
}

void FileBase::Close() const
{
    if (_file) fclose(_file);
    _file = NULL;
}
void FileBase::Reopen() const
{
    Open();
}

void FileBase::Open() const
{
    if (_file) Close();
    std::string bs = (_isBinary) ? "b" : "";
    switch (_mode)
    {
        case QIODevice::ReadOnly:
            _file = fopen(Path().c_str(), std::string("r"+bs).c_str());
            break;
        case QIODevice::WriteOnly:
    {
        std::string path = Path();
            _file = fopen(Path().c_str(), std::string("w"+bs).c_str());
            break;
     }   case QIODevice::Append:
            _file = fopen(Path().c_str(), std::string("a"+bs).c_str());
            break;
        default:
            break;
    }
    if (!_file)
    {
        Log( Path() + ", " + std::string(strerror(errno)) );
        throw BadFileName("FileBase::Open", Path());
    }
    QFile* f = new QFile();
    f->open(_file, _mode);
    _ds.setDevice(f);
}
