#ifndef FILEBASE_H
#define FILEBASE_H

#include "../globals/globals.h"
#include "../globals/log.h"
#include "../memrep/wavedata.h"

#include <stdio.h> //Wanted to do this just with Qt streaming, didn't
    //seem to work well

class FileBase
{
    public:
        virtual ~FileBase();

        static std::string MakeTempFileName(const std::string& users_dir, const std::string& user_name, const std::string& input_file_name, wv::TRANSFORM transform,
                                 wv::CLUSTER_ID cid, const std::string& type, int electrode);
        static std::string MakeTempFileName(const std::string& full_path, const std::string& input_file_name, wv::TRANSFORM transform,
                                 wv::CLUSTER_ID cid, const std::string& type, int electrode);
        static void TempFileComponents(const std::string& temp_file, std::string& user_dir,
                                 std::string& name, wv::TRANSFORM& transform, wv::CLUSTER_ID& cid, int& eletrode);
        //Convenience. **Note that these functions expect whatever folder is supplied to contain a 'temp' directory, in which
        //the file is to be found/put.  'type' is wfm, tim, or clu

        static void SetUsersDir(const std::string& users_dir) { _usersDir = users_dir; }
        static void SetUserName(const std::string& user_name) { _userName = user_name; }
        static void SetUserDataDir(const std::string& user_data_dir) { _userDataDir = user_data_dir; }
        static void SetUserWSDataDir(const std::string& user_wsdata_dir) { _userWSDataDir = user_wsdata_dir; }

        void Close() const; //For input files that need object to persist, or when file has to be closed
            //before being reopened in the same scope (SPC_TO::LoadClassification).
        void Reopen() const;

        static std::string UsersDir() { return _usersDir; }
        static std::string UserName() { return _userName; }
        static std::string UserPath() { return _usersDir + "/" + _userName; }
        static std::string UserDataDir() { return _userDataDir; }
        static std::string UserWSDataDir() { return _userWSDataDir; }

        bool IsNull() const { return _file==NULL; }
        std::string Path() const { return _dir + "/" + _name; }
        const std::string& Name() const { return _name; }
        const std::string& Dir() const { return _dir; }

        //Note that electrode number is not saved int the wfm file.  The data out of a single wfm file is assumed to
        //come from a single electrode.
        //  This is the file format used for all WE temporary files, for one thing.
        struct WfmHeader
        {
            uchar type; //Whether electrode data, timestamps, or neither are stored along with the waveforms
            int num_records,
                num_samples;
            uchar bps, //bytes per sample
                is_integral; //1 for integral type, 0 for floating point
            wv::float_t scale, shift;
                //For e.g. the case where floating point data has been compressed into shorts.  All types are
                //assumed to be unsigned.
        };

    protected:
        FileBase(const std::string& name, QIODevice::OpenMode mode, bool is_binary);
        QDataStream& DS() { return _ds; }
        FILE* File() const { return _file; }

    private:
        void Open() const;

        static std::string _usersDir, _userName,
                _userDataDir, _userWSDataDir; //These two are *absolute paths*

        std::string _name, //Changing the name will be allowed
                _dir;
        QIODevice::OpenMode _mode;
        bool _isBinary;
        mutable QDataStream _ds;
        mutable FILE* _file; //So that const functions can still use it
};

//typedef FileBase FB;

#endif // FILEBASE_H
