#include "casedatafilein.h"

CaseDataFileIn::CaseDataFileIn(const std::string& full_path, int num_records, wv::float_t pval)
    : InputFileBase(full_path, false), _numRecords(num_records), _pVal(pval)
{
}
CaseDataFileIn::~CaseDataFileIn()
{
    delete static_cast<Classification*>( Data() );
}

int CaseDataFileIn::Load()
{
    FILE* fp = File();
    if (!fp)
    {
        Log("WARNING: Invalid file name, " + Path() + ", no records loaded.");
        return -1;
    }
    QFile qfp;
    qfp.open(fp, QIODevice::ReadOnly);

    std::string line = qfp.readLine().data();
    while ( !ut::Contains(line, "Case#") )
        line = qfp.readLine().data();
            //Stupidly, AutoClass doesn't simply just give you the data, it puts it in a file
            //with extra text

    wv_vector_int* vcluster = wv_vector_int_alloc(_numRecords);
    for (int i=0; i<_numRecords; ++i)
    {
        line = qfp.readLine().data();
        StringVec tokens;
        boost::algorithm::split(tokens, line, boost::algorithm::is_any_of(" \t"),
                                boost::algorithm::token_compress_on);
        int raw_c = boost::lexical_cast<int>( tokens.at(1) );
        wv::float_t probability = boost::lexical_cast<wv::float_t>( tokens.at(2) );
        int c = (probability>=_pVal) ? raw_c : wv::NULL_CLUSTER;
        wv_vector_int_set(vcluster, i, c);
    }

    //Now create the classification
    SetData( new Classification(vcluster) );

    return _numRecords;
}
