#ifndef INPUTFILEBASE_H
#define INPUTFILEBASE_H

#include "../../memrep/classification.h"
#include "../../memrep/parameters/profile.h"
#include "../filebase.h"

typedef std::pair<wv_vector_int*, wv_matrix*> KMPair;

class InputFileBase : public FileBase
{
    public:
        virtual ~InputFileBase() override {}

        virtual int Load() = 0; //Returns # of records read

        int NumSamples() const { return _numSamples; }
        WaveData* WData(short electrode) const;
        wv_matrix* GData() const { return static_cast<wv_matrix*>(_data); }  //For reading temporary files
            //This gets used use non-const contexts, can't be const
        const Classification* CData() const { return static_cast<const Classification*>(_data); }
        const std::map<int, KMPair>* KMData() const
        {
            return static_cast<const std::map<int, KMPair>* >(_data);
        }
            //This is for KMeans file
        const Profile* PData() const { return static_cast<const Profile*>(_data); }

        std::vector<short> VElectrodes() const;
        const std::set<short>& Electrodes() const { return _electrodes; }
        int NumElectrodes() const { return _electrodes.size(); }

    protected:
        InputFileBase(const std::string& name, bool is_binary);
        void AddElectrode(short e) { _electrodes.insert(e); }
        void SetNumSamples(int num_samples) { _numSamples = num_samples; }
        void SetWData(short electrode, WaveData* wave_data);
        void SetData(void* data) { _data = data; }
        void* Data() { return _data; }

    private:
        void* _data; //See the ?Data getter functions above for what this can.
        std::set<short> _electrodes;
        int _numSamples;
};

typedef std::shared_ptr<InputFileBase> InputFilePtr;

#endif // INPUTFILEBASE_H
