#include "kmeansfilein.h"

KMeansFileIn::KMeansFileIn(const std::string& file_name, int num_clusters, int num_records, int num_samples)
    : InputFileBase(file_name, true), _numClusterings(num_clusters), _numRecords(num_records)
{
    SetNumSamples(num_samples);
}
KMeansFileIn::~KMeansFileIn()
{
    std::map<int, KMPair>* data = static_cast< std::map<int, KMPair>* >( Data() );
    if (data)
    {
        for (auto it : *data)
        {
            wv_vector_int_free(it.second.first);
            wv_matrix_free(it.second.second);
        }
        delete data;
    }
}

int KMeansFileIn::Load()
{
    std::map<int, KMPair>* data = new std::map<int, KMPair>();
    FILE* fp = File();
    if (!fp)
    {
        Log("ERROR: Could not open KMeans output file.");
        return 0;
    }
    for (int i=0; i<_numClusterings; ++i)
    {
        int k;
        fread(&k, sizeof(int), 1, fp);
        wv_vector_int* assignment = wv_vector_int_alloc(_numRecords);
        wv_vector_int_fread(fp, assignment);
        wv_matrix* centers = wv_matrix_alloc( k, NumSamples() );
        wv_matrix_fread(fp, centers);

        KMPair kmpair(assignment, centers);
        (*data)[k] = kmpair;
    }
//    while ( !feof(fp) );
    SetData(data);
    return _numRecords;
}
