#include "spcfilein.h"

const int SPCFileIn::NUM_HEADER_COLS = 4; //The first column is rank, the second is
//temperature, the third is always 0 (?), the 4th is unclustered spikes, or something

SPCFileIn::SPCFileIn(const std::string& file_name)
    : InputFileBase(file_name + ".dg_01.lab", false), _fileNameBase(file_name), _clusterCounts(NULL)
{
}
SPCFileIn::~SPCFileIn()
{
    std::vector<ClusPair>* data = static_cast< std::vector<ClusPair>* >( Data() );
    if (data)
    {
        for (auto it=data->begin(); it!=data->end(); ++it)
            wv_vector_int_free(it->second);
        delete data;
    }
}
int SPCFileIn::Load()
{
    //Get the output parameters from the .param file
    _outputParams.clear();
    std::string param_file = _fileNameBase + ".param";
    QFile pf(param_file.c_str());
    pf.open(QFile::ReadOnly);
    if (!File()) throw BadFileName("SPCFileIn::Load", Path());
    while (!pf.atEnd())
    {
        std::string line = pf.readLine().data();
        StringVec tokens = ut::Split<StringVec>(line, "\t");
        std::string key = tokens.at(0),
                value = tokens.at(1);
        boost::algorithm::trim(key);
        boost::algorithm::trim(value); //Get rid of '\n'
        _outputParams[key] = value;
    }
    pf.close();

    //Construct the data containers
    int num_records = boost::lexical_cast<int>(_outputParams["NumberOfPoints"]);
    std::vector<ClusPair>* clusters = new std::vector<ClusPair>();

    //Read the SPC output clustering file
    if (!File()) throw BadFileName("SPCFileIn::Load", Path());
    QFile fp;
    fp.open(File(), QFile::ReadOnly);
    if (!fp.isOpen())  throw BadFileName("SPCFileIn::Load", Path());
    while (!fp.atEnd())
    {
        std::string line = fp.readLine().data();
        boost::algorithm::trim(line);
        StringVec tokens;
        boost::split(tokens, line, boost::is_any_of(" "), boost::token_compress_on);
        wv::float_t temp = boost::lexical_cast<wv::float_t>( tokens.at(1) );
        wv_vector_int* vcluster = wv_vector_int_alloc(num_records);
        for (int i=0; i<num_records; ++i)
        {
            int c = boost::lexical_cast<int>( tokens.at(i+2) );
            wv_vector_int_set(vcluster, i, c);
        }
        clusters->push_back( ClusPair(temp, vcluster) );
    }
    fp.close();

    SetData(clusters);

    //Now get the N-per-cluster, for each temperature, from the .dg_01 file
    _outputParams.clear();
    std::string nclus_file = _fileNameBase + ".dg_01";
    QFile ncf(nclus_file.c_str());
    ncf.open(QFile::ReadOnly);
    if (_clusterCounts)
    {
        wv_matrix_int_free(_clusterCounts);
        _clusterCounts = NULL;
    }
    int i=0, num_top_clus(INT_MAX);
    while (!ncf.atEnd())
    {
        std::string line = ncf.readLine().data();
        boost::algorithm::trim(line);
        StringVec tokens;
        boost::split(tokens, line, boost::is_any_of(" "), boost::token_compress_on);
        if (!_clusterCounts)
        {
            num_top_clus = tokens.size() - NUM_HEADER_COLS;
            _clusterCounts = wv_matrix_int_alloc(clusters->size(), num_top_clus);
        }
        for (int j=0; j<num_top_clus; ++j)
            wv_matrix_int_set(_clusterCounts, i, j,
                               boost::lexical_cast<int>( tokens.at(j+NUM_HEADER_COLS) ));
        ++i;
    }
    ncf.close();

    return num_records;
}

wv::float_t SPCFileIn::Temperature(int i) //const
{
    const std::vector<ClusPair>* clusters = static_cast< std::vector<ClusPair>* >( Data() );
    ClusPair cluster = clusters->at(i);
    return cluster.first;
}
const wv_vector_int* SPCFileIn::VCluster(int i) //const
{
    const std::vector<ClusPair>* clusters = static_cast< std::vector<ClusPair>* >( Data() );
    ClusPair cluster = clusters->at(i);
    return cluster.second;
}

