#include "userclufilein.h"

UserCluFileIn::UserCluFileIn(const std::string &full_path)
    : InputFileBase(full_path, false)
{
}

int UserCluFileIn::Load()
{
    throw WIOException("UserCluFileIn::Load", Name(), "Must take bitset argument");
}

void UserCluFileIn::Load(const boost::dynamic_bitset<>& is_scrubbed)
{
    int num_records = (~is_scrubbed).count();
    wv_vector_int* vcluster(NULL), * vcluster_all(NULL);
        //The *_all suffix denotes structures which include scrubbed waveforms

    //Now get the cluster values, both putting them in the big num_records-length
    FILE* clu = File();
    if (clu)
    {
        fscanf(clu, "%d\n", &_numRecordsAll);
        if (is_scrubbed.size() != (size_t)_numRecordsAll)
            throw WIOException("MainWindow::OnActionLoadClassificationTriggered",
                               Name(), "File corrupted or was not derived from this input file, cannot load.");
        vcluster_all = wv_vector_int_alloc(_numRecordsAll);

        char clus[1024];
        fgets(clus, 1024, clu); //These are not needed for WaveSorter, so we don't do anything with
            //them here

        int code = wv_vector_int_fscanf(clu, vcluster_all);
        if (code)
            Log("ERROR: Did not read cluster temporary file correctly");

        vcluster = wv_vector_int_alloc(num_records);
        int ct=0;
        for (int i=0; i<_numRecordsAll; ++i)
        {
            int clus = wv_vector_int_get(vcluster_all, i);
            if (clus != wv::NULL_CLUSTER || !is_scrubbed.test(i))
                wv_vector_int_set(vcluster, ct++, clus);
        }
    }
    else
    {
        Log("WARNING: Invalid file name, " + Path() + ", no records loaded.");
        return;
    }

    //Now create the classification
    SetData( new Classification(vcluster) );

    //Free the larger data vector
    wv_vector_int_free(vcluster_all);
}
