#ifndef WAVEFORMSINBASE_H
#define WAVEFORMSINBASE_H

#include "../inputfilebase.h"

class WaveformsInBase : public InputFileBase
{
    friend class Batch;
    friend class ChannelBatch;
    friend class MainWindow;

    public:
        static const int UTAH_ARRAY_96; //Number of electrodes

        static WaveformsInBase* Create(const std::string& name);
        virtual ~WaveformsInBase() override;

        void SetIsBatch(bool is_batch) { _isBatch = is_batch; }

        bool IsBatch() const { return _isBatch; }
        int NumWaves(int i) { return _numWaves.at(i); }
        wv::float_t UsPerSample() const { return _usPerSample; }

    protected:
        WaveformsInBase(const std::string& full_path, bool is_binary);

        void FinishedLoading();
        int GenerateTransforms(short elc = -1);
        void MakeWaveData(const std::vector<short>& v_electrodes, bool has_online_clus);
            //Create a wavedata object for each electrode.
            //This function requires a vector which containing the electrode number associated with each waveform
        void SetUsPerSample(wv::float_t us) { _usPerSample = us; }

        QProgressBar* _progressBar; //Too convenient to make private, for now

    private:
        bool _isBatch;
        std::vector<int> _numWaves;
        wv::float_t _usPerSample; //microseconds per sample
};

typedef std::shared_ptr<WaveformsInBase> WavesFilePtr;

#endif // WAVEFORMSINBASE_H
