#include "mclufileout.h"

MCluFileOut::MCluFileOut(const std::string& file_name, const Classification* cls)
    : OutputFileBase(file_name, true, NULL, boost::dynamic_bitset<>()), _best(cls->BestLevel())
{
    std::vector<Clustering> clusterings = cls->VClusters();
    int num_clusterings = clusterings.size();
    for (int i=0; i<num_clusterings; ++i)
    {
        const wv_vector_int* clustering = clusterings.at(i).second;
        _clusters.push_back( ConstClustering(clusterings.at(i).first, clustering) );
    }
}
MCluFileOut::MCluFileOut(const std::string& file_name, int best,
                         const std::vector<ConstClustering>& clusterings)
    : OutputFileBase(file_name, true, NULL, boost::dynamic_bitset<>()), _best(best), _clusters(clusterings)
{

}
MCluFileOut::MCluFileOut(const std::string& file_name, int best, const wv_vector_int* vcluster)
    : OutputFileBase(file_name, true, NULL, boost::dynamic_bitset<>()), _best(best)
{
    ConstClustering clustering(0.0, vcluster);
    _clusters.push_back(clustering);
}

void MCluFileOut::Save() const
{
    if (_clusters.empty())
    {
        Log("ERROR: Classification empty, no records saved.");
        return;
    }

    FILE* clu = File();
    if (clu)
    {
        int num_records = _clusters.at(0).second->size,
                num_clusterings = _clusters.size();

        fwrite(&num_records, sizeof(int), 1, clu);
        fwrite(&num_clusterings, sizeof(int), 1, clu);
        fwrite(&_best, sizeof(int), 1, clu);

        for (int i=0; i<num_clusterings; ++i)
        {
            wv::float_t level = _clusters.at(i).first;
            const wv_vector_int* clustering = _clusters.at(i).second;
            fwrite(&level, sizeof(wv::float_t), 1, clu);
            wv_vector_int_fwrite(clu, clustering);
        }
    }
    else
        Log("WARNING: Invalid file name, " + Name() + ", no records saved.");
}
