#include "simplebinary.h"

const int SimpleBinary::BYTE_SZ = 8 * sizeof(uchar);
SimpleBinary::SimpleBinary(const std::string& name, const WaveData* data, const boost::dynamic_bitset<>& accepted)
    : OutputFileBase(name, true, data, accepted)
{
    const boost::dynamic_bitset<>& is_scrubbed = Data()->Scrubbed(); //This has the size of the *original* number of
        //records.
    int num_records = Data()->NumRecords();
    _numAllRecords = num_records + is_scrubbed.count();
    bool need_padding = num_records%BYTE_SZ==0;
    _numBytes = (need_padding) ? _numAllRecords/BYTE_SZ : _numAllRecords/BYTE_SZ + 1;
    _byteArray = new uchar[_numBytes];
    int byte_idx=0;
    uchar current_byte = 0;
    int ct=0; //This ct is for *non-scrubbed* records, which is what IsAccepted indexes.
    for (int i=0; i<_numAllRecords; ++i)
    {
        current_byte <<= 1;
        if (!is_scrubbed.test(i))
            current_byte |= (uchar)(IsAccepted(ct++));
        if ((i+1)%BYTE_SZ==0)
        {
            _byteArray[byte_idx++] = current_byte;
            current_byte = 0;
        }
    }
    if (need_padding) //In this case, the last byte didn't get filled up in the loop
        _byteArray[byte_idx] = current_byte;
}

void SimpleBinary::Save() const
{
    FILE* file = File();
    fwrite(&_numAllRecords, sizeof(int), 1, file);
    fwrite(&_byteArray, BYTE_SZ/8, _numBytes, file);
}
