#include "electrodegui.h"
#include "ui_electrodegui.h"
#include "../../globals/globals.h"

ElectrodeGui::ElectrodeGui(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::ElectrodeGui)
{
    ui->setupUi(this);
//    layout()->setSizeConstraint( QLayout::SetFixedSize );
//    ui->layButtons->setSizeConstraint( QLayout::SetFixedSize );
        //###These don't work b/c the gui is within a MDI

}

ElectrodeGui::~ElectrodeGui()
{
    delete ui;
}

void ElectrodeGui::SetElectrodes(WavesFilePtr waves_file)
{
    _wavesFile = waves_file;

    const std::string& file_name = waves_file->Name();
    ui->lblFile->setText( file_name.c_str() );

    const std::vector<short>& electrodes = _wavesFile->VElectrodes();
    int num_electrodes = electrodes.size();

    //Erase old buttons
    ui->lsElectrodes->clear();

    //Add new buttons
    for (int i=0; i<num_electrodes; ++i)
    {
        short elc = electrodes.at(i);
        ui->lsElectrodes->addItem( ("Channel " + std::to_string(elc)).c_str() );
    }
    ui->lsElectrodes->setCurrentRow(0);

    update();
}
void ElectrodeGui::SetElectrodeButton(short elc)
{
    int idx = ut::IndexOf( _wavesFile->VElectrodes(), elc);
    ui->lsElectrodes->setCurrentRow(idx);
}

short ElectrodeGui::Electrode() const
{
    uint row = (short)ui->lsElectrodes->currentRow();
    if (row>=_wavesFile->VElectrodes().size())
        throw BadGuiOp("ElectrodeGui::Electrode", "No electrode in range is selected");
    return _electrodes.at(row);
}

void ElectrodeGui::OnLsElectrodesCurrentRowChanged(int i, bool)
{
    if (i==-1) return; //When we've cleared the list
    short elc = _wavesFile->VElectrodes().at(i);
    emit SetElectrode(elc, _wavesFile);
        //MainWindow::ElectrodeSet ignores this signal if the electrode is the same as
        //what is already current.
}
