#include "paramgui.h"
#include "ui_paramgui.h"

ParamGui::ParamGui(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::ParamGui), _pars(NULL), _isParamsChanged(false)
{
    ui->setupUi(this);
}
ParamGui::~ParamGui()
{
    delete ui;
}

void ParamGui::SetParameters(std::shared_ptr<ParamsBase> pars)
{
    if (!pars)
        throw BadGuiOp("ParamGui::SetParameters", "Null Parameters");
    _pars = pars;
    _isParamsChanged = true;
    setWindowTitle(_pars->Name().c_str());
    update();
}

void ParamGui::paintEvent(QPaintEvent*)
{
    if (!_isParamsChanged) return;
    ui->tblPars->clear();
    ui->tblPars->setRowCount(0);
    ui->tblPars->setHorizontalHeaderItem(0, new QTableWidgetItem("Value"));
    int num_rows = _pars->NumPars();
    for (int i=0, ct=0; i<num_rows; ++i)
    {
        if (_pars->IsInternal( _pars->Key(i) )) continue;
        ui->tblPars->insertRow(ct);
        ui->tblPars->setVerticalHeaderItem(ct, new QTableWidgetItem(_pars->Key(i).c_str()));
        ui->tblPars->setItem(ct,0, new QTableWidgetItem(_pars->Value(i).c_str()));
        ++ct;
    }
    _isParamsChanged = false;
}

void ParamGui::OnBtnDefaultsClicked(bool)
{
    _pars->RestoreDefaults();
    _isParamsChanged = true;
    update();
}
void ParamGui::OnBtnSaveToProfileClicked(bool)
{
    const ParamVec& param_list = std::static_pointer_cast<const ParamsBase>(_pars)->ParList();

    ProfileOut profile_out(_pars->Name(), param_list);
    profile_out.Save();
    profile_out.Close();

    //Now reload the profile, so that changes are duly noted (particularly for the Session Make X parameters).
    ProfileIn profile_in;
    profile_in.Load();
    session_profile = *profile_in.PData(); //The Profile in ProfileIn is deleted when
        //it goes out of scope
}
void ParamGui::on_tblPars_itemChanged(QTableWidgetItem* item)
{
    std::string val = item->text().toStdString();
    int i = item->row();
    std::string key = ui->tblPars->verticalHeaderItem(i)->text().toStdString();
    _pars->SetPar(key, val);
}
