#ifndef WAVEGUI_H
#define WAVEGUI_H

#include "../../memrep/wavedata.h"
#include "../../memrep/classification.h"
#include "../../memrep/wavetemplate.h"
#include "../widgets/wave/cluswavewidget.h"
#include "../../analysis/dunn.h"
#include "../gui.h"

//To draw the waves, WaveGui needs the raw data *and* the classification, but
//nothing more--not the file and not the analysis.

namespace Ui
{
    class WaveGui;
}

class ClusterMediator;
class WaveGui : public QWidget
{
    Q_OBJECT

    public:
        static int DEFAULT_NUM_SHOWN;

        explicit WaveGui(wv::CLUSTER_ID cid);
        ~WaveGui();

        void SetTemplates();
        void Setup();
            //It is not until *this* function is called that all of the graphics etc. get generated.  And of course this
            //has to wait until the analysis is actually complete.
            //  ***This function does *not* call SetWaves(), that has to be called independently
            // When cls == NULL, the function is basically being used to choose a new level from the existing classification
            //(via slider movement).
        void SetWaves();
        void SetViewable(const boost::dynamic_bitset<>& is_viewable);

        const Ui::WaveGui* GetUi() const { return ui; }
        static const std::vector<QColor>& ColorPalette();
        const std::vector<QColor>& Colors() const { return _colors; }
        bool HaveFirstAccepted() const { return _haveFirstAccepted; }

    signals:
        void Closed(wv::CLUSTER_ID cid);
        void GotFocus(wv::CLUSTER_ID cid);
        void ResetWaves(); //Get rid of this when full colors are used
        void SendClusters(wv::CLUSTER_ID cid);

    public slots:
        void on_cboxRelative_clicked(bool is_checked);

    protected:
        void closeEvent(QCloseEvent*);

    private slots:
        void on_btnAccept_clicked() { gui::Slot(this, &WaveGui::OnBtnAcceptClicked); }
        void on_btnAnalyze_clicked() { gui::Slot(this, &WaveGui::OnBtnAnalyzeClicked); }
        void on_btnStats_clicked() { gui::Slot(this, &WaveGui::OnBtnStats); }
        void on_btnTemplates_clicked() { gui::Slot(this, &WaveGui::OnBtnTemplatesClicked); }
        void LevelSliderMoved(int value);
        void UpdateShown();
        void UpdateWavePlot();

    private:
        void OnBtnAcceptClicked(bool is_prog = true);
        void OnBtnAnalyzeClicked(bool is_prog = true);
        void OnBtnStats(bool is_prog = true);
        void OnBtnTemplatesClicked(bool is_prog = true);

        void CalcShown();
        void CalcClusAccepted();
        static void MakeColorPalette();
        void ResetButtons();
        void UpdateIsCBoxShown();

        Ui::WaveGui* ui;
        const wv::CLUSTER_ID _cid;
        ClusterMediator& _cmed;
        static std::vector<QColor> _colorPalette;
        std::vector<QColor> _colors; //The former is the rotation of colors used,
            //The latter is the color association for each waveform
        std::map<int,bool> _haveTemplates; //Only want to calculate these once
        std::map<int,bool> _isBoxShown;
            //Which checkboxes are selected, indexed by gui number.
            // *All visible clusters will be saved as 'accepted' spikes when
            //the Accept button is pressed
        int _numWavesShown; //Generally can show only a subset of all waves.
        std::vector<int> _percents; //What percent of the *total* set of waveforms belong in each
            //cluster

        std::map<int, int> _clusterIndexes; //The gui order (starting with 0) of the clusters
        boost::dynamic_bitset<> _isViewable; //Waveforms to be displayed in _waveWidget, subject to further checkbox
                                    //subselection.  Takes _windowFilter into account.  This is a (typically small)
                                    // **subset** of all accepted waveforms.
        bool _haveFirstAccepted; //It's annoying to have the widget automatically send an 'everything is
            //accepted' message when it first comes into focus, before the user has actually accepted
            //anything
};

typedef std::shared_ptr<WaveGui> WaveGuiPtr;

#endif // WAVES_H
