#ifndef WAVESPACEGUI_H
#define WAVESPACEGUI_H

#include "../globals/globals.h"
#include "../memrep/clustermediator.h"
#include "../memrep/specs/wavespaceguispec.h"
#include "../memrep/specs/scalespec.h"
#include "widgets/wave/scanwavewidget.h"
#include "widgets/histwidget.h"
#include "../analysis/threads/mgr/histthreadmgr.h"
#include "widgets/labelwidget.h"
#include "../memrep/wavedata.h"
#include "../memrep/wavetemplate.h"
#include "../memrep/parameters/cluster/manualpars.h"
#include "../memrep/parameters/cluster/autoclasspars.h"
#include "../memrep/parameters/cluster/kmeanspars.h"
#include "../memrep/parameters/cluster/spcparameters.h"
#include "../memrep/parameters/cluster/kkparameters.h"
#include "../memrep/drawing/polyline.h"
#include "gui.h"

#include "../file/output/profileout.h"

namespace Ui {
    class WaveSpaceGui;
}

class WaveSpaceGui : public QWidget
{
    Q_OBJECT

    public:
        explicit WaveSpaceGui(QWidget* parent);
        ~WaveSpaceGui();

        void InitializeSpecs(wv::CLUSTER_ID cid); //For every cid this needs to be set up, whether
            //it's shown on the WaveSpaceGui or not
        void Setup(wv::CLUSTER_ID cid);
        void UpdateAll(bool reset_samples = false); //Assumes the _wsgSpec is complete, updates everything about the gui

        int WindowBegin() const { return _cmed.GetCluSpecPtr(wv::DEFAULT_CID)->GetWSGSpec().WindowBegin(); }
        int WindowEnd() const { return _cmed.GetCluSpecPtr(wv::DEFAULT_CID)->GetWSGSpec().WindowEnd(); }
            //These report the range of waveforms which will be submitted for clustering.
            //These values correspond to *record number*, not pixels or anything like that.
        LabelWidget* AxisLabel(WSGSpec::AXIS axis) const;

    signals:
        void SendParameters(ParamsBase* pars);
        void UpdateWavesShown() const;

    public slots:
        void ReceiveClusters(wv::CLUSTER_ID cid);
        void HistWheelMoved(bool is_forward);
        void ResetZoomMode(); //see HistWidget
        void RecalculateScatter(); //see HistWidget
        void EnableTransform(wv::TRANSFORM transform, short electrode);
        void ReceiveDragEvent();
        void ReceiveToggle(int scrub_index);
        void StopMovie();

    protected:
        void resizeEvent(QResizeEvent*);

    private slots:
        void on_cboxTime_clicked(bool value)
        {
            gui::Slot2(this,  std::bind(&WaveSpaceGui::OnCboxTimeClicked, this, value, std::placeholders::_1) );
        }
        void TimeSliderMoved(int value);
        void HistSliderMoved(int value);
        void HistDiskMoved(int selected_disk, wv::float_t x, wv::float_t y);

        void WavesSliderMoved(int value);
        void WavesRangeChanged(int begin, int end); //From ScanWaveWidget

        void EdWindowChanged();
        void EdNumShownChanged();
        void WindowSliderMoved(int value);
        void NumShownSliderMoved(int value);

        void on_edNumShown_editingFinished();

        void on_rbtnHistAxis_clicked();
        void on_rbtnHistResolution_clicked();
        void on_rbtnHistLuminance_clicked();
        void on_btnScatter_clicked();
        void on_btnMovieGui_clicked();

        void on_btnPlay_clicked() { gui::Slot(this, &WaveSpaceGui::OnBtnPlayClicked); }
        void NextFrame();

        void on_rbtnSetWavesFromXAxis_clicked() { gui::Slot(this, &WaveSpaceGui::OnRbtnSetWavesFromXAxisClicked); }
        void on_rbtnSetWavesFromYAxis_clicked() { gui::Slot(this, &WaveSpaceGui::OnRbtnSetWavesFromYAxisClicked); }
        void on_rbtnSetWavesFromRaw_clicked() { gui::Slot(this, &WaveSpaceGui::OnRbtnSetWavesFromRawClicked); }
            //The 3 radio buttons at the top of the WaveWidget, which enable the user
            //to shift scanning modes without having to re-select transforms.

        void on_rbtnBar_clicked();
        void on_rbtnDisk_clicked();

        void on_btnSetHistAxisX_clicked() { gui::Slot(this, &WaveSpaceGui::OnBtnSetHistAxisXClicked); }
        void on_btnSetHistAxisY_clicked() { gui::Slot(this, &WaveSpaceGui::OnBtnSetHistAxisYClicked); }
        void on_btnAddToUser_clicked() { gui::Slot(this, &WaveSpaceGui::OnBtnAddToUserClicked); }
        void on_btnRemoveFromUser_clicked() { gui::Slot(this, &WaveSpaceGui::OnBtnRemoveFromUserClicked); }

        void on_rbtnUser_clicked() { gui::Slot(this, &WaveSpaceGui::OnRbtnUserClicked); }
        void on_rbtnTimestamps_clicked() { gui::Slot(this, &WaveSpaceGui::OnRbtnTimestampsClicked); }
        void on_rbtnHeight_clicked() { gui::Slot(this, &WaveSpaceGui::OnRbtnHeightClicked); }
        void on_rbtnWidth_clicked() { gui::Slot(this, &WaveSpaceGui::OnRbtnWidthClicked); }
        void on_rbtnSlope_clicked() { gui::Slot(this, &WaveSpaceGui::OnRbtnSlopeClicked); }
        void on_rbtnSlopeD_clicked() { gui::Slot(this, &WaveSpaceGui::OnRbtnSlopeDClicked); }
        void on_rbtnEnergy_clicked() { gui::Slot(this, &WaveSpaceGui::OnRbtnEnergyClicked); }
        void on_rbtnRaw_clicked() { gui::Slot(this, &WaveSpaceGui::OnRbtnRawClicked); }
        void on_rbtnRawD_clicked() { gui::Slot(this, &WaveSpaceGui::OnRbtnRawDClicked); }
#ifdef USE_GSL
        void on_rbtnSpline_clicked() { gui::Slot(this, &WaveSpaceGui::OnRbtnSplineClicked); }
        void on_rbtnSplineD_clicked() { gui::Slot(this, &WaveSpaceGui::OnRbtnSplineDClicked); }
        void on_rbtnFourier_clicked() { gui::Slot(this, &WaveSpaceGui::OnRbtnFourierClicked); }
        void on_rbtnFourierD_clicked() { gui::Slot(this, &WaveSpaceGui::OnRbtnFourierDClicked); }
#endif
        void on_rbtnWavelets_clicked() { gui::Slot(this, &WaveSpaceGui::OnRbtnWaveletsClicked); }
        void on_rbtnWaveletsD_clicked() { gui::Slot(this, &WaveSpaceGui::OnRbtnWaveletsDClicked); }
        void on_rbtnPCA_clicked() { gui::Slot(this, &WaveSpaceGui::OnRbtnPCAClicked); }
        void on_rbtnPCAD_clicked() { gui::Slot(this, &WaveSpaceGui::OnRbtnPCADClicked); }

        void on_btnNewUserTransform_clicked() { gui::Slot(this, &WaveSpaceGui::OnBtnNewUserTransformClicked); }
//        void on_btnSaveUserTransforms_clicked() { gui::Slot(this, &WaveSpaceGui::OnBtnSaveUserTransformsClicked); }
        void on_lsUTContents_itemClicked(QListWidgetItem* item)
        {
            gui::Slot2(this,  std::bind(&WaveSpaceGui::OnLsUTContentsItemClicked, this, item, std::placeholders::_1) );
        }
        void on_spnUserTransform_valueChanged(int value)
        {
            gui::Slot2(this,  std::bind(&WaveSpaceGui::OnSpnUserTransformValueChanged, this, value, std::placeholders::_1) );
        }

        void on_btnTemplates_clicked() { gui::Slot(this, &WaveSpaceGui::OnBtnTemplatesClicked); }

        void on_btnSetSamples_clicked() { gui::Slot(this, &WaveSpaceGui::OnBtnSetSamplesClicked); }

        void on_btnCluster_clicked() { gui::Slot(this, &WaveSpaceGui::OnBtnClusterClicked); }
        void on_btnResetClassification_clicked() { gui::Slot(this, &WaveSpaceGui::OnBtnResetClassificationClicked); }

        void on_rbtnManual_clicked() { gui::Slot(this, &WaveSpaceGui::OnRbtnManualClicked); }
        void on_rbtnAutoClass_clicked() { gui::Slot(this, &WaveSpaceGui::OnRbtnAutoClassClicked); }
        void on_rbtnKMeans_clicked() { gui::Slot(this, &WaveSpaceGui::OnRbtnKMeansClicked); }
        void on_rbtnSPC_clicked() { gui::Slot(this, &WaveSpaceGui::OnRbtnSPCClicked); }
        void on_rbtnKK_clicked() { gui::Slot(this, &WaveSpaceGui::OnRbtnKKClicked); }

        void on_edSliderValue_editingFinished();
        void on_cmbDrawLines_activated(const QString& text)
        {
            gui::Slot2(this,  std::bind(&WaveSpaceGui::OnCmbDrawLinesActivated, this,
                               text, std::placeholders::_1) );
        }
        void on_btnScrub_clicked() { gui::Slot(this, &WaveSpaceGui::OnBtnScrubClicked); }
        void on_btnEllipse_clicked() { gui::Slot(this, &WaveSpaceGui::OnBtnBiEllipseClicked); }
        void on_spnSD_valueChanged(double val)
        {
            gui::Slot2(this,  std::bind(&WaveSpaceGui::OnSpnSDValueChanged, this, val, std::placeholders::_1) );
        }
        void on_btnCenter_clicked() { gui::Slot(this, &WaveSpaceGui::OnBtnCenterClicked); }
        void on_btnCircle_clicked() { gui::Slot(this, &WaveSpaceGui::OnBtnCircleClicked); }
        void on_btnPolylineUp_clicked() { gui::Slot(this, &WaveSpaceGui::OnBtnPolylineUpClicked); }
        void on_btnPolylineDown_clicked() { gui::Slot(this, &WaveSpaceGui::OnBtnPolylineDownClicked); }
        void on_btnClearDrawline_clicked() { gui::Slot(this, &WaveSpaceGui::OnBtnClearDrawLine); }

    private:

        static const int DEFAULT_NUM_XBINS; //How many records go into a single x bin, on average
        static const int DEFAULT_NUM_YBINS; //How many records go into a single y bin, on average

        void OnCboxTimeClicked(bool value, bool is_prog = true);
        void OnBtnPlayClicked(bool is_prog = true);
        void OnRbtnSetWavesFromXAxisClicked(bool is_prog = true);
        void OnRbtnSetWavesFromYAxisClicked(bool is_prog = true);
        void OnRbtnSetWavesFromRawClicked(bool is_prog = true);
        void OnBtnSetHistAxisXClicked(bool is_prog = true);
        void OnBtnSetHistAxisYClicked(bool is_prog = true);
        void OnBtnNewUserTransformClicked(bool is_prog = true);
        void OnBtnSaveUserTransformsClicked(bool is_prog = true); //Saves the whole profile, currently
        void OnBtnAddToUserClicked(bool is_prog = true);
        void OnBtnRemoveFromUserClicked(bool is_prog = true);
        void OnBtnTemplatesClicked(bool is_prog = true);
        void OnBtnSetSamplesClicked(bool is_prog = true);
        void OnBtnClusterClicked(bool is_prog = true);
        void OnBtnResetClassificationClicked(bool is_prog = true);

        void OnRbtnUserClicked(bool is_prog = true);
        void OnRbtnTimestampsClicked(bool is_prog = true);
        void OnRbtnHeightClicked(bool is_prog = true);
        void OnRbtnWidthClicked(bool is_prog = true);
        void OnRbtnSlopeClicked(bool is_prog = true);
        void OnRbtnSlopeDClicked(bool is_prog = true);
        void OnRbtnEnergyClicked(bool is_prog = true);
        void OnRbtnRawClicked(bool is_prog = true);
        void OnRbtnRawDClicked(bool is_prog = true);
        void OnRbtnSplineClicked(bool is_prog = true);
        void OnRbtnSplineDClicked(bool is_prog = true);
        void OnRbtnFourierClicked(bool is_prog = true);
        void OnRbtnFourierDClicked(bool is_prog = true);
        void OnRbtnWaveletsClicked(bool is_prog = true);
        void OnRbtnWaveletsDClicked(bool is_prog = true);
        void OnRbtnPCAClicked(bool is_prog = true);
        void OnRbtnPCADClicked(bool is_prog = true);        
        void OnLsUTContentsNewIndex(int index);
        void OnLsUTContentsItemClicked(QListWidgetItem* item, bool is_prog = true);
        void OnSpnUserTransformValueChanged(int value, bool is_prog = true);

        void OnRbtnManualClicked(bool is_prog = true);
        void OnRbtnAutoClassClicked(bool is_prog = true);
        void OnRbtnKMeansClicked(bool is_prog = true);
        void OnRbtnSPCClicked(bool is_prog = true);
        void OnRbtnKKClicked(bool is_prog = true);

        void OnCmbDrawLinesActivated(const QString& text, bool is_prog = true);
        void OnBtnScrubClicked(bool is_prog = true);
        void OnBtnBiEllipseClicked(bool is_prog = true);
        void OnSpnSDValueChanged(wv::float_t sd, bool is_prog = true);
        void OnBtnCenterClicked(bool is_prog = true);
        void OnBtnCircleClicked(bool is_prog = true);
        void OnBtnPolylineUpClicked(bool is_prog = true);
        void OnBtnPolylineDownClicked(bool is_prog = true);
        void OnBtnClearDrawLine(bool is_prog = true);

        void MakeBAandColors(int num_records); //From _clusAccepted.  BA = bit array, this does green vs. white
        void StartPolyline(Polyline::REJECT_DIR reject_dir);
        void ResetClassification(); //Gets rid of green vs. white, circles and polylines
        void ResetDiskCursors();
        void SafeUnload(); //Need to reset views in WSGSpec, since pointers have been invalidated!!
        void ToggleWave(int scrub_index);

        void SetHistCursor(HIST_CURSOR_MODE cursor_mode);
        void SetHistAxis(WSGSpec::AXIS axis);
        void SetDrawButtons(bool is_shown);
        void SetMovieGui(bool is_shown, bool is_update = true);
        void SetUserControls(bool is_shown);

        void UpdateHistogram();
        void UpdateWaves(); //The purpose of this function is to set _isWaveShown
        void UpdateTimeSlider();
        void UpdateHistWidget(bool reset_window = false); //This does pure gui updating, e.g. enabling/disabling sliders.  Changes
            //_histActiveWidget
        void UpdateWavesWidget(bool reset_samples = false) const; //This does pure gui updating, e.g. enabling/disabling sliders
        void UpdateUTContents();

        void MakeTemplates();
        void FinishDrawline(bool stop = true);
            //Finishes the current drawline, whatever that is.  Resets HistWidget::_drawLineState by default,
            //not done in the case of an ellipse
        void ResetDrawGui();
        void ClearDrawLines();
        void ClearCurrentDrawLines();
        void DrawScanline() const;
        void DrawHistWindow(bool reset_window = false) const;
        void InitCircle();

        Ui::WaveSpaceGui* ui;

        //These are all held in WaveSpaceGui or ClusterMediator
        CluSpecPtr _cluSpec;
        ClusterMediator& _cmed;
        WaveData* _waveData;
        WSGSpec* _wsgSpec;
            //Used so frequently, might as well make them members.  Set *only* in Setup

        //Everything here is used only here in WaveSpaceGui
        std::vector<QColor> _allColors;
        std::vector<int> _clusters; //The individual cluster numbers, derived from _clusAccepted.
        wv_histogram2d* _histAccept,
                        * _histUnaccept; //Needs to be members so that they can be deleted after painting
            //Note that *all* waves are represented in the histogram; the unaccepted ones are white,
            //the accepted ones are green
        wv_matrix_int* _matAccept, * _matUnaccept;
        bool _needNewTemplates; //Constructing templates is time-consuming so we build them only when necessary.
        int _numXBins, _numYBins, _prevNumXBins, _prevNumYBins;
        ScaleSpec* _scaleSpec;
            //ScaleSpec lives here and not in WSGSpec because it changes too rapidly, its state is not
            //worth preserving
        QTimer _timer;
        bool _waitForEllipse; //Make the SD spinbox available
};

#endif // WAVESPACEGUI_H
