#include "classification.h"

Classification::Classification(int best_level, const std::vector<Clustering>& clusterings)
    : _bestLevel(best_level), _vClusterings(clusterings)
{
    int num_clusterings = _vClusterings.size();
    for (int i=0; i<num_clusterings; ++i)
        AddClusterSet(i);
}
Classification::Classification(wv_vector_int* vcluster) : _bestLevel(0)
{
    _vClusterings.push_back( Clustering(0.0, vcluster) );
    AddClusterSet(0);
}
Classification::~Classification()
{
    int num_clusterings = _vClusterings.size();
    for (int i=0; i<num_clusterings; ++i)
        wv_vector_int_free( _vClusterings.at(i).second );
}

void Classification::AddClusterSet(int i)
{
    const Clustering& clustering = _vClusterings.at(i);
    wv_vector_int* vclus = clustering.second;
    int num_records = vclus->size;
    std::set<int> selected_set;
    for (int i=0; i<num_records; ++i)
    {
        int c = wv_vector_int_get(vclus, i);
        if (c==wv::NULL_CLUSTER) continue;
        selected_set.insert(c);
    }
    std::vector<int> clus = ut::ToVector(selected_set);
    qSort(clus.begin(), clus.end()); //std::set members have no predictable order

    _clusters.push_back(clus);
}
