#ifndef CLASSIFICATION_H
#define CLASSIFICATION_H

#include "../globals/globals.h"
#include "../globals/log.h"

//So, the classification keeps track of the classification of every waveform.  But
//ultimately only waveforms from *1* cluster can get written to file.  (May relax
//later but this is the idea).

//There should NOT be derived classes here.  The classification, once achieved,
//should be independent of the analysis used to derive it, *and should not be modified*.
//Everything modifiable should go in ResultSpec.

typedef std::pair<wv::float_t, wv_vector_int*> Clustering;
typedef std::pair<wv::float_t, const wv_vector_int*> ConstClustering;

class Classification
{
    public:
        Classification(int best_level, const std::vector<Clustering>& clusterings);
        Classification(wv_vector_int* vcluster);
        ~Classification();

        int BestLevel() const { return _bestLevel; } //The *index*
        const std::vector< std::vector<int> >& Clusters() const { return _clusters; } //clusterings
        const std::vector<Clustering>& VClusters() const { return _vClusterings; }
        const std::vector<int>& Clusters(int level_idx) const { return _clusters.at(level_idx); }
            //The clusters comprising a particular clustering
        int NumClusters(int level_idx) const { return _clusters.at(level_idx).size(); }
        bool IsValid() const { return _vClusterings.size() > 0; }

    private:
        void AddClusterSet(int i);

        const int _bestLevel; //This is what is returned by the clustering algorithm
        std::vector< std::vector<int> > _clusters;
        std::vector<Clustering> _vClusterings;
};

typedef std::shared_ptr<Classification> ClassificationPtr;
typedef std::shared_ptr<const Classification> ClassificationCPtr;

#endif // CLASSIFICATION_H
