#include "drawlinebase.h"

int DrawLineBase::_drawLineIDCt = 0;
const int DrawLineBase::DRAWLINE_MAXLEN = 2048;

DrawLineBase::DrawLineBase(ID id)
    : _id(id), _isFinished(false), _length(0)
{
    _points = new QPointF[DRAWLINE_MAXLEN];
}
DrawLineBase::DrawLineBase()
    : _id(-1), _isFinished(false), _length(0), _points(NULL)
{
}
DrawLineBase::DrawLineBase(const DrawLineBase& drawline)
{
    Copy(drawline);
}
DrawLineBase& DrawLineBase::operator=(const DrawLineBase& drawline)
{
    if (this != &drawline) Copy(drawline);
    return *this;
}
void DrawLineBase::Copy(const DrawLineBase& drawline)
{
    _id = drawline._id;
    _isFinished = drawline._isFinished;
    _length = drawline._length;

    _points = new QPointF[DRAWLINE_MAXLEN];
    for (int i=0; i<_length; ++i)
    {
        _points[i].setX( drawline._points[i].x() );
        _points[i].setY( drawline._points[i].y() );
    }
}
DrawLineBase::~DrawLineBase()
{
    if (_points) delete[] _points;
}

DrawLineBase::ID DrawLineBase::NewDrawLineID()
{
    return _drawLineIDCt++;
}

void DrawLineBase::AddPoint(QPointF point)
{
    if (_length==DRAWLINE_MAXLEN) throw BadDrawLine("DrawLineBase::AddPoint", "Too many points");

    _points[_length] = point;
    ++_length;
}

QPointF DrawLineBase::Point(int i) const
{
    return _points[i];
}


DrawLineBaseSpec::DrawLineBaseSpec(const DrawLineBaseSpec& dlbs)
{
    draw_line       = (dlbs.draw_line) ? dlbs.draw_line->Clone() : nullptr;
    transform_x     = dlbs.transform_x;
    transform_y     = dlbs.transform_y;
    sample_x        = dlbs.sample_x;
    sample_y        = dlbs.sample_y;
    time_begin      = dlbs.time_begin;
    time_end        = dlbs.time_end;
}

DrawLineBase* DrawLineBaseSpec::GetDrawLine()
{
    return draw_line;
}
const DrawLineBase* DrawLineBaseSpec::GetDrawLine() const
{
    return draw_line;
}
std::string DrawLineBaseSpec::Name() const
{
    char cname[256];
    sprintf(cname, "%s %d, %s %d",  wv::TransformName(transform_x).c_str(), sample_x,
            wv::TransformName(transform_y).c_str(), sample_y);
    std::string name(cname);
    return name;
}
