#ifndef POLYLINE_H
#define POLYLINE_H

#include "drawlinebase.h"

class Polyline : public DrawLineBase
{
    public:
#ifndef USE_GSL
        static const int LUT_LENGTH;
#endif

        enum REJECT_DIR
        {
            REJECT_UP,
            REJECT_DOWN
        };

        Polyline(DrawLineBase::ID id, REJECT_DIR reject_dir);
        Polyline();
        Polyline(const Polyline& polyline); //Need a deep copy, for drawing
        Polyline& operator=(const Polyline& polyline);
        virtual DrawLineBase* Clone() override { return new Polyline(*this); }
        virtual ~Polyline() override;

        virtual void AddPoint( QPointF point ) override; //The point is specified in pixel coordinates
        REJECT_DIR RejectDir() const { return _rejectDir; }
        wv::float_t Value(wv::float_t x) const;

        // ***This is the function that returns the interpolated value.

    private:
        void MakeSpline();
        REJECT_DIR _rejectDir;
#ifdef USE_GSL
        gsl_interp_accel* _accel;
        gsl_spline* _spline;
#else
        wv::float_t* _lut; //Lookup table for super-fast interpolation.  It's
            //just an extremely finely sampled version of the polyline.
#endif
};

struct PolylineSpec : public DrawLineBaseSpec
{
    PolylineSpec(wv::TRANSFORM tr_x, wv::TRANSFORM tr_y, int s_x, int s_y, Polyline* polyline)
        : DrawLineBaseSpec(tr_x, tr_y, s_x, s_y, polyline)
    {
    }
    PolylineSpec() {}
    virtual ~PolylineSpec() override {}

    virtual Polyline* GetDrawLine() override
    {
        return static_cast<Polyline*>( DrawLineBaseSpec::GetDrawLine() );
    }
    virtual const Polyline* GetDrawLine() const override
    {
        return static_cast<const Polyline*>( DrawLineBaseSpec::GetDrawLine() );
    }
};
typedef PolylineSpec PLSpec;
    //NB, NO tid, user transforms not allowed with Manual sorting.

#endif // POLYLINE_H
