#include "histthreadmgr.h"
#include "histto.h"

HistThreadMgr::HistThreadMgr(const Spec::HistSpec& hist_spec) : ThreadMgr(qBound(1, QThread::idealThreadCount()-1, (int)hist_spec.xdata_vec->size), 4000),
    _histSpec(hist_spec), _histAccept(NULL), _histUnaccept(NULL)
{
    int num_records = _histSpec.xdata_vec->size;
    _recordsPerThread = num_records / NumThreads();
}

WorkerTO* HistThreadMgr::CreateWorkerTO(int thd_num)
{
    int start_row = thd_num*_recordsPerThread,
            num_records = _histSpec.xdata_vec->size;
    Spec::HistSpec hist_spec = _histSpec;
    hist_spec.start_row = start_row;
    hist_spec.end_row = (thd_num==NumThreads()-1) ? num_records : start_row+_recordsPerThread;
    HistTO* to = new HistTO(thd_num, hist_spec);
    return to;
}

void HistThreadMgr::OnThreadFinished(void* result)
{
    auto results_pair = static_cast< QPair<gsl_histogram2d*, gsl_histogram2d*>* >(result);
    gsl_histogram2d* hist_accept = results_pair->first,
            * hist_unaccept = results_pair->second;

    if (_histAccept)
    {
        gsl_histogram2d_add(_histAccept, hist_accept);
        gsl_histogram2d_add(_histUnaccept, hist_unaccept);

        gsl_histogram2d_free(hist_accept);
        gsl_histogram2d_free(hist_unaccept);
    }
    else
    {
        _histAccept = hist_accept;
        _histUnaccept = hist_unaccept;
    }

    delete results_pair;
}

