#include "histto.h"

HistTO::HistTO(int thd_num, Spec::HistSpec hist_spec)
    : WorkerTO(thd_num), _histSpec(hist_spec)
{
}

void HistTO::Start()
{
    //Get information from _histSpec, mostly for convenience.
    int num_bins_x = _histSpec.num_bins_x,
        num_bins_y = _histSpec.num_bins_y,
        start_row = _histSpec.start_row,
        end_row = _histSpec.end_row;
    double min_x = _histSpec.min_x,
            min_y = _histSpec.min_y,
            max_x = _histSpec.max_x,
            max_y = _histSpec.max_y;
    const QBitArray& is_accepted = _histSpec.is_accepted;
    const WaveSpaceGuiSpec& spec = _histSpec.spec;
    const gsl_vector* xdata_vec = _histSpec.xdata_vec,
            * ydata_vec = _histSpec.ydata_vec;

    //Allocate and initialize histograms
    gsl_histogram2d* hist_accept = gsl_histogram2d_alloc(num_bins_x, num_bins_y),
                * hist_unaccept = gsl_histogram2d_alloc(num_bins_x, num_bins_y);
    gsl_histogram2d_set_ranges_uniform(hist_accept, min_x, max_x, min_y, max_y);
    gsl_histogram2d_set_ranges_uniform(hist_unaccept, min_x, max_x, min_y, max_y);

    bool accept_nonempty = !is_accepted.isEmpty(),
            use_transform = spec.HistLimitMode()==Spec::TRANSFORM;
    double time_begin = spec.TimeBegin(),
            time_end = spec.TimeEnd();
    for (int i=start_row; i<end_row; ++i)
    {
        double timestamp = spec.Timestamp(i);

        if (use_transform
                || (timestamp >= time_begin && timestamp <= time_end))
        {
            double xval = gsl_vector_get(xdata_vec, i),
                    yval = gsl_vector_get(ydata_vec, i);

            if (accept_nonempty && is_accepted.testBit(i))
                gsl_histogram2d_increment(hist_accept, xval, yval);
            else
                gsl_histogram2d_increment(hist_unaccept, xval, yval);
        }
            //Get it--if the record falls outside the time window, it isn't included in the histogram.
    }

    auto result = new QPair<gsl_histogram2d*, gsl_histogram2d*>(hist_accept, hist_unaccept);
    WorkFinished(result);
}
