#include "clusterparamsbase.h"
#include "manualpars.h"
#include "autoclasspars.h"
#include "kmeanspars.h"
#include "spcparameters.h"
#include "kkparameters.h"
#include "../../../memrep/clustermediator.h"

ClusterParamsBase::ClusterParamsBase(const std::string& name) : ParamsBase(name)
{
}

ClusterParamsPtr ClusterParamsBase::Create(wv::CLUSTER_METHOD method)
{
    ClusterParamsBase* parameters(NULL);
    switch (method)
    {
        case wv::MANUAL:
            parameters = new ManualPars();
            break;
        case wv::AUTOCLASS:
            parameters = new AutoClassPars();
            break;
        case wv::KMEANS:
            parameters = new KMeansPars();
            break;
        case wv::KK:
            parameters = new KKParameters();
            break;
        case wv::SPC:
            parameters = new SPCParameters();
            break;
        default:
           break;
    }
    if (!parameters) throw BadClusterMethod("ClusterParamsBase::Create", method);
    return ClusterParamsPtr( parameters );
}

void ClusterParamsBase::SetSampleRangePars()
{
    CluSpecPtr clu_spec = ClusterMediator::Instance().GetCluSpecPtr(wv::DEFAULT_CID);
    int begin = clu_spec->SelectedSamplesBegin(),
            end = clu_spec->SelectedSamplesEnd();
    SetPar("samples_begin", std::to_string(begin));
    SetPar("samples_end", std::to_string(end));
}

