#include "clusterspec.h"
#include "../clustermediator.h"

ClusterSpec::ClusterSpec()
    : _clusterMethod(wv::UNKNOWN_METHOD), _selectedSamplesBegin(-1), _selectedSamplesEnd(-1),
      _tid(wv::NO_TRANSFORM_ID), _transform(wv::UNKNOWN_TRANSFORM), _waveData(nullptr)
{
}
ClusterSpec::~ClusterSpec()
{
    DeleteDrawLines();
}

CluSpecPtr ClusterSpec::Clone() const
{
    ClusterSpec* clu_spec( new ClusterSpec );

    clu_spec->_clusterMethod =          _clusterMethod;
    clu_spec->_clusterParameters =      _clusterParameters;
    clu_spec->_priorAccepted =          _priorAccepted;
    clu_spec->_selectedSamplesBegin =   _selectedSamplesBegin;
    clu_spec->_selectedSamplesEnd =     _selectedSamplesEnd;
    clu_spec->_tid =                    _tid;
    clu_spec->_transform =              _transform;
    clu_spec->_toggledWaves =           _toggledWaves;
    clu_spec->_waveData =               _waveData;
    clu_spec->_wsgSpec =                _wsgSpec;

    for (auto it : _centers)
    {
        CenterSpec* center_spec = new CenterSpec(*it);
        clu_spec->_centers.push_back(center_spec);
    }
    for (auto it : _circles)
    {
        CSpec* c_spec = new CSpec(*it);
        clu_spec->_circles.push_back(c_spec);
    }
    for (auto it : _polylines)
    {
        PLSpec* pl_spec = new PLSpec(*it);
        clu_spec->_polylines.push_back(pl_spec);
    }

    return CluSpecPtr(clu_spec);
}

//Most notably, WSGSpec is left out of this, as are all values that pertain to manual clustering
CluSpecPtr ClusterSpec::MintNew() const
{
    ClusterSpec* clu_spec( new ClusterSpec );

    clu_spec->_clusterMethod =          _clusterMethod;
    clu_spec->_clusterParameters =      _clusterParameters;
    clu_spec->_selectedSamplesBegin =   _selectedSamplesBegin;
    clu_spec->_selectedSamplesEnd =     _selectedSamplesEnd;
    clu_spec->_tid =                    _tid;
    clu_spec->_transform =              _transform;
    clu_spec->_waveData =               _waveData;

    return CluSpecPtr(clu_spec);
}

void ClusterSpec::SetWaveData(WaveData* wave_data)
{
    _waveData = wave_data;
    _wsgSpec.SetWaveData(wave_data);
}

void ClusterSpec::InitToggledWaves()
{
    _toggledWaves = boost::dynamic_bitset<>( Waves()->size1 );
}
void ClusterSpec::InitWaves(wv::CLUSTER_ID cid)
{
    ClusterMediator& cmed = ClusterMediator::Instance();
    _waveData = cmed.GetWaveData(cid);
    _toggledWaves = boost::dynamic_bitset<>( Waves()->size1 );

    //In addition it's necessary to initialize some WaveSpaceGuiSpec members
    _wsgSpec.SetWavesTransform(_transform, cid);
    _wsgSpec._window = boost::dynamic_bitset<>( Waves()->size1 );
}

int ClusterSpec::NumSamples() const
{
    if (!Waves()) throw NullTransform("ClusterSpec::NumSamples");
    return Waves()->size2;
}

MatCPtr ClusterSpec::Waves() const
{
    if (_waveData)
        return _waveData->Waveforms(_transform, _tid);
    return MatCPtr();
}

void ClusterSpec::SetPriorAccepted(const boost::dynamic_bitset<>& accepted)
{
    if (_toggledWaves.count() == 0)
        _priorAccepted = accepted;
    else
        _priorAccepted = _toggledWaves ^= accepted;
}
void ClusterSpec::SetTransform()
{
    SetTransform( _wsgSpec.Transform( _wsgSpec.ActiveAxis() ),
                  _wsgSpec.TransformId( _wsgSpec.ActiveAxis() ) );
}
void ClusterSpec::SetTransform(wv::TRANSFORM transform, wv::TRANSFORM_ID tid)
{
    _transform = transform;
    _tid = tid;
}

void ClusterSpec::DeleteDrawLines()
{
    for (auto it : _centers) delete it;
    for (auto it : _circles) delete it;
    for (auto it : _polylines) delete it;
    _centers.clear();
    _circles.clear();
    _polylines.clear();
}
