#ifndef SCALESPEC_H
#define SCALESPEC_H

#include "../../globals/globals.h"

//The value data here are native data coordinates, zooming already taken into
//account.  I.e. these must represent the limits of the values that are *visible*.
class ScaleSpec
{
    public:
        ScaleSpec(wv::float_t x1_val, wv::float_t x2_val, wv::float_t y1_val, wv::float_t y2_val,
                  wv::float_t x1_px, wv::float_t x2_px, wv::float_t y1_px, wv::float_t y2_px);
        ScaleSpec();

        void SetValuesX(wv::float_t x_min_val, wv::float_t x_max_val);
        void SetValuesY(wv::float_t y_min_val, wv::float_t y_max_val);
        void SetValuesPix(wv::float_t x_min_px, wv::float_t x_max_px, wv::float_t y_min_px, wv::float_t y_max_px);

        wv::float_t MaxPixX() const { return _xMaxPix; }
        wv::float_t MaxPixY() const { return _yMaxPix; }
        wv::float_t MaxValX() const { return _xMaxVal; }
        wv::float_t MinValX() const { return _xMinVal; }
        wv::float_t MaxValY() const { return _yMaxVal; }
        wv::float_t MinValY() const { return _yMinVal; }

        QPointF PixToVal(QPointF pix) const;
        wv::float_t PixToValX(wv::float_t xpix) const;
        void PixToValXVec(wv_vector* xpix_new, const wv_vector* xpix_old = NULL) const;
        wv::float_t PixToValY(wv::float_t ypix) const;
        void PixToValYVec(wv_vector* ypix_new, const wv_vector* ypix_old = NULL) const;

        QPointF ValToPix(QPointF val) const;
        wv::float_t ValToPixX(wv::float_t x) const;
        void ValToPixXVec(wv_vector* xpix_new, const wv_vector* xpix_old = NULL) const;
        wv::float_t ValToPixY(wv::float_t y) const;
        void ValToPixYVec(wv_vector* ypix_new, const wv_vector* ypix_old = NULL) const;

    private:
#ifndef Q_OS_WIN
        ScaleSpec(const ScaleSpec&) = delete;
        ScaleSpec& operator=(const ScaleSpec&) = delete;
            //So basically it's a singleton--Drawline objects get its address, which
            //is set to null when they are kicked off the WSG into a manual cluster
#else
        ScaleSpec(const ScaleSpec&);
        ScaleSpec& operator=(const ScaleSpec&);
#endif
        wv::float_t _xMinVal, _xMaxVal, _yMinVal, _yMaxVal;
        wv::float_t _xMinPix, _xMaxPix, _yMinPix, _yMaxPix; //Pixel values
};

#endif // SCALESPEC_H
